//-----------------------------------------------------------------------------
// NOTE: Sample only for linux side cause of the used select!
//-----------------------------------------------------------------------------
#ifndef linux
#	error Sample only for linux side!!
#endif	// linux
#include <stdio.h>
#include <unistd.h>
#include <iostream>
#include <mvIMPACT_CPP/mvIMPACT_acquire.h>

#ifdef MALLOC_TRACE
#	include <mcheck.h>
#endif	// MALLOC_TRACE

#define PRESS_A_KEY				\
	cout << "Press a key..." << endl;	\
	getchar();

using namespace std;
using namespace mvIMPACT::acquire;

//-----------------------------------------------------------------------------
// returns 0 if timeout, else 1
unsigned WaitForInput( int maxwait_sec, int fd )
//-----------------------------------------------------------------------------
{
    fd_set rfds;
    struct timeval tv;

    FD_ZERO(&rfds);
    FD_SET(fd, &rfds);

    tv.tv_sec = maxwait_sec ;
    tv.tv_usec = 0;

    return select(fd+1, &rfds, NULL, NULL, &tv);
}

//-----------------------------------------------------------------------------
ostream& operator<<( std::ostream& out, const Property& prop )
//-----------------------------------------------------------------------------
{
	out << prop.name() << ": " << prop.readS();
	return out;
}

//-----------------------------------------------------------------------------
unsigned int liveLoop( Device* pDev, const string& settingName = "" )
//-----------------------------------------------------------------------------
{
	unsigned int cnt = 0;

	cout << " == " << __FUNCTION__ << " - Initialising the device. This might take some time...." << endl;
	try
	{
		pDev->open();
	}
	catch( ImpactAcquireException& e )
	{
		// this e.g. might happen if the same device is already opened in another process...
		cout << "*** " << __FUNCTION__ << " - An error occurred while opening the device " << pDev->serial.read()
			 << "(error code: " << e.getErrorCode() << ", " << e.getErrorCodeAsString() << "). Press any key to end the application..." << endl;
		PRESS_A_KEY
		return 0;
	}

	cout << " == " << __FUNCTION__ << " - establish access to the statistic properties...." << endl;
	// establish access to the statistic properties
	Statistics statistics( pDev );
	cout << " == " << __FUNCTION__ << " - create an interface to the device found...." << endl;
	// create an interface to the device found
	FunctionInterface fi( pDev );

	if( !settingName.empty() )
	{
		cout << "Trying to load setting " << settingName << "..." << endl;
		int result = fi.loadSetting( settingName );
		if( result != DMR_NO_ERROR )
		{
			cout << "loadSetting( \"" << settingName << "\" ); call failed: " << ImpactAcquireException::getErrorCodeAsString( result ) << endl;
		}
	}

	SystemSettings ss(pDev);
	int MAX_REQUESTS = 4;
	if( ss.requestCount.isConstDefined( plMaxValue ) )
	{
		MAX_REQUESTS = ss.requestCount.read( plMaxValue );
	}
	// make sure enough requests are avaiable:
	fi.ensureRequests( ( MAX_REQUESTS < 10 ) ? MAX_REQUESTS : 10 );
	// now we can work with at least MAX_REQUESTS requests

#if 0
	// if running mvBlueFOX on an embedded system (e.g. ARM) with USB 1.1 it may be necessary to change
	// a few settings and timeouts like this:

	// get other settings
	SettingsBlueFOX setting( pDev );

	// set request timeout higher because USB 1.1 on ARM is soooo slow
	setting.cameraSetting.imageRequestTimeout_ms.write( 5000 );
	// use on Demand mode
	setting.cameraSetting.triggerMode.write( ctmOnDemand );
	// use no image processing - just raw data
	setting.imageProcessing.colorProcessing.write( cpmRaw );
#endif

#if 0
	// this section contains special settings that might be interessting for mvBlueCOUGAR or mvBlueLYNX M7
	// related embedded devices
	CameraSettingsBlueCOUGAR cs(pDev);
	int maxWidth = cs.aoiWidth.read( plMaxValue );
	cs.aoiWidth.write( maxWidth );
// 	cs.autoGainControl.write( agcOff );
// 	cs.autoExposeControl.write( aecOff );
	cs.exposeMode.write( cemOverlapped );
 	cs.pixelClock_KHz.write( cpc40000KHz );
// 	cs.expose_us.write( 5000 );
	
	// If this is color sensor, we will NOT convert the Bayer data into a RGB image as this
	// will cost a lot of time on an embedded system
	ImageProcessing ip(pDev);
	ip.colorProcessing.write( cpmRaw );

	// There are two modes to operate the memory manager in. A flexible an a fast mode.
	// As long as we don't change the size of the image with each snap, we will use the fast
	// one
	ImageMemoryManager imm(pDev);
	imm.mode.write( mmmPool );
#endif

	// prefill the capture queue. There can be more then 1 queue, but for this sample
	// we will work with the default capture queue
	for( int i=0; i<MAX_REQUESTS; i++ )
	{
		try
		{
			fi.imageRequestSingle( );
		}
		catch( ImpactAcquireException& e )
		{
			cout << "***" << __FUNCTION__ << " " << e.getErrorString().c_str() <<"("<< e.getErrorCode() <<")"<< endl;
		}
	}

	cout << "Press <<ENTER>> to end the application!!" << endl;

	// run thread loop
	const Request* pRequest = 0;
	const unsigned int timeout_ms = 8000;	// USB 1.1 on an embedded system needs a large timeout for the first image
	int requestNr = -1;
	bool boLoopRunning = true;
	while( boLoopRunning )
	{
		// wait for results from the default capture queue
		requestNr = fi.imageRequestWaitFor( timeout_ms );

		if( fi.isRequestNrValid( requestNr ) )
		{
			pRequest = fi.getRequest( requestNr );
			if( fi.isRequestOK( pRequest ) )
			{
				++cnt;
				// here we can display some statistical information every 10th image
				if( cnt%10 == 0 )
				{
					cout << cnt << ": Info from " << pDev->serial.read()
						<< ": " << statistics.framesPerSecond << ", "
						<< statistics.errorCount << ", " << statistics.captureTime_s <<" Image count: " << cnt 
						<< " (dimensions: " << pRequest->imageWidth.read() << "x" << pRequest->imageHeight.read()
						<< ")" << endl;
				}
			}
			else
			{
				cout << "*** Error: " << pRequest->requestResult << endl;
			}

			// this image has been displayed thus the buffer is no longer needed...
			fi.imageRequestUnlock( requestNr );
			// send a new image request into the capture queue
			fi.imageRequestSingle( );
		}
		else
		{
			cout << "*** Error: There has been no request in the queue!" << endl;
		}

		boLoopRunning = WaitForInput ( 0, STDIN_FILENO ) == 0 ? true : false;	// break by STDIN
	}

	cout << " == " << __FUNCTION__ << " - free resources...." << endl;

	// free resources
	fi.imageRequestReset( 0, 0 );
	return 0;
}

//-----------------------------------------------------------------------------
Device* getDeviceFromUserInput( const DeviceManager& devMgr )
//-----------------------------------------------------------------------------
{
	unsigned int devCnt = devMgr.deviceCount();
	if(devCnt == 0 )
	{
		cout << "No MATRIX VISION device found!" << endl;
		return 0;
	}

	// display every device detected
	for( unsigned int i=0; i<devCnt; i++ )
	{
		Device* pDev = devMgr[i];
		if( pDev )
		{
			cout << "[" << i << "]: " << pDev->serial.read() 
				<< " (" << pDev->product.read() << ", family: "
				<< pDev->family.read() << ")" << endl;
		}
	}

	// get user input
	cout << endl << "Please enter the number in front of the listed device followed by [ENTER] to open it: ";
	unsigned int devNr = 0;
	cin >> devNr;
	// remove the '\n' from the stream
	cin.get();

	if( devNr >= devCnt )
	{
		cout << "Invalid selection!" << endl;
		return 0;
	}

	cout << "Using device number " << devNr << "." << endl;
	return devMgr[devNr];
}

//-----------------------------------------------------------------------------
int main( int argc, char* argv[] )
//-----------------------------------------------------------------------------
{
#ifdef MALLOC_TRACE
	mtrace();
#endif	// MALLOC_TRACE

	cout << " ++ start application...." << endl;

	DeviceManager devMgr;
	Device* pDev = getDeviceFromUserInput( devMgr );
	if( pDev == 0 )
	{
		cout << "Unable to continue!";
		cout << "Press [ENTER] to end the application" << endl;
		cin.get();
		return 0;
	}

	string settingName;
	if( argc > 1 )
	{
		// try to load this setting later on...
		settingName = string(argv[1]);
	}

	// create an interface to the first MATRIX VISION device with the serial number sDevSerial
	if( pDev )
	{
		cout << "Initialising the device: "<< pDev->serial.read() << ". This might take some time..." << endl;
		// start the execution of the 'live' loop.
		liveLoop( pDev, settingName );
		cout << " == Will exit...." << endl;
		pDev->close();
		// do NOT delete pDev here! It will be destroyed automatically when the device manager (devMgr) is destroyed
	}
	cout << " -- end application...." << endl;

	return 0;
}

