//-----------------------------------------------------------------------------
#include "DeviceConfigureFrame.h"
#include "DeviceHandlerBlueFOX.h"
#include "KernelDriverUpdate.h"
#include <sstream>
#include "wx/msgdlg.h"
#include <apps/Common/wxAbstraction.h>

using namespace std;

//-----------------------------------------------------------------------------
bool DeviceHandlerBlueFOX::SupportsKernelDriverUpdate( bool& boNewerDriverAvailable, std::string& kernelDriverName )
//-----------------------------------------------------------------------------
{
	return SupportsKernelDriverFeature( pDev_, kernelDriverName, boNewerDriverAvailable );
}

//-----------------------------------------------------------------------------
int DeviceHandlerBlueFOX::UpdateFirmware( void )
//-----------------------------------------------------------------------------
{
	ConvertedString serial(pDev_->serial.read());
	wxMessageDialog AreYouSureDlg( NULL,
		wxString::Format( wxT("Are you sure you want to update the firmware of device %s?\n"), serial.c_str() ),
		wxT("Firmware update"),
		wxNO_DEFAULT | wxYES_NO | wxICON_INFORMATION );

	if( AreYouSureDlg.ShowModal() == wxID_YES )
	{
		wxMessageDialog infoDlg( NULL,
			wxT("The firmware will now be updated. During this time(approx. 30 sec.) the application will not react. Please be patient.\n"),
			wxT("information"),
			wxOK | wxICON_INFORMATION );
		infoDlg.ShowModal();

		pParent_->WriteLogMessage( wxString::Format( wxT( "Updating firmware of device %s. This might take some time. Please be patient.\n"), serial.c_str() ) );
		int result = pDev_->updateFirmware();
		if( result == DMR_FEATURE_NOT_AVAILABLE )
		{
			pParent_->WriteLogMessage( wxT( "This device doesn't support firmware updates.\n") );
		}
		else if( result != DMR_NO_ERROR )
		{
			pParent_->WriteLogMessage( wxString::Format( wxT( "An error occurred: %s(please refer to the manual for this error code).\n"), ConvertedString(ImpactAcquireException::getErrorCodeAsString( result )).c_str() ) );
		}
		else
		{
			pParent_->WriteLogMessage( wxString::Format( wxT("%s.\n"), ConvertedString(pDev_->HWUpdateResult.readS()).c_str() ) );
		}
		if( pDev_->HWUpdateResult.read() == urUpdateFWOK )
		{
			wxString str(wxT("Update successful. Please disconnect and reconnect the device now to activate the new firmware.") );
			wxMessageDialog updateOKDlg( NULL,
				str,
				wxT("Update result"),
				wxOK | wxICON_INFORMATION );
			updateOKDlg.ShowModal();
		}
		return urOperationSuccessful;
	}
	else
	{
		pParent_->WriteLogMessage( wxString::Format( wxT("Firmware update canceled for device %s.\n"), ConvertedString(serial).c_str() ) );
		return urOperationCanceled;
	}
}

//-----------------------------------------------------------------------------
int DeviceHandlerBlueFOX::UpdateKernelDriver( void )
//-----------------------------------------------------------------------------
{
	wxMessageDialog AreYouSureDlg( pParent_,
								   wxString::Format( wxT("Are you sure you want to update the kernel driver for every device belonging to the\n%s family that is currently connected?\n"), ConvertedString(pDev_->family.read()).c_str() ),
								   wxT("Kernel driver update"),
								   wxNO_DEFAULT | wxYES_NO | wxICON_INFORMATION );
	if( AreYouSureDlg.ShowModal() == wxID_YES )
	{
		ostringstream resultMsg;
		wxString str;
		
		if( ::UpdateKernelDriver( pDev_->family.read(), resultMsg ) != kdurOK )
		{
			wxTextAttr redStyle(wxColor(255, 0, 0));
			str = wxT("Failed to update kernel driver. Message: ");
			str.append( ConvertedString(resultMsg.str()) );
			pParent_->WriteLogMessage( str, redStyle );
		}
		else
		{
			str = wxT("Update successful.");
		}
		wxMessageDialog updateDlg(	pParent_,
									str,
									wxT("Update result"),
									wxOK | wxICON_INFORMATION );
		updateDlg.ShowModal();
		return urOperationSuccessful;
	}
	else
	{
		pParent_->WriteLogMessage( wxString::Format( wxT("Kernel driver update for %s devices canceled.\n"), ConvertedString(pDev_->family.read()).c_str() ) );
		return urOperationCanceled;
	}
}


